<?php
/**
 * Plugin Name:     Issues taxonomy
 * Plugin URI:      https://rene.seindal.dk/wordpress/issues
 * Description:     Creates a taxonomy for editorial issues
 * Author:          René Seindal
 * Author URI:      https://rene.seindal.dk/
 * Text Domain:     issues-taxonomy
 * Domain Path:     /languages
 * Version:         1.5
 *
 * @package         IssuesTaxonomy
 */

namespace ReneSeindal;

require_once( ABSPATH . 'wp-admin/includes/taxonomy.php' );

class IssuesPlugin {
    // Defaults
    protected $taxonomy = 'issue';
    protected $post_types = [ 'post' ];

    /*************************************************************
     *
     * Automatic setup of actions, filters and shortcodes based on
     * available class methods
     *
     *************************************************************/

	function __construct() {
        $methods = get_class_methods( $this );
        foreach ( $methods as $m ) {
            if ( preg_match( '/^do_(.*)_shortcode$/', $m, $match ) ) {
                add_shortcode( $match[1], [ $this, $m ] );
            }
            elseif ( preg_match( '/^do_(.*)_action(_.*)?/', $m, $match ) ) {
                $action = $match[1];
                $extra = $match[2] ?? '';
                $prio = $this->callback_param( "$action$extra", 'priority', 10);
                $argc = $this->callback_param( $action, 'arguments', 1);
                add_action( $action, [ $this, $m ], $prio, $argc );
            }
            elseif ( preg_match( '/^do_(.*)_filter(_.*)?/', $m, $match ) ) {
                $filter = $match[1];
                $extra = $match[2] ?? '';
                $prio = $this->callback_param( "$filter$extra", 'priority', 10);
                $argc = $this->callback_param( $filter, 'arguments', 1);
                add_filter( $filter, [ $this, $m ], $prio, $argc );
            }
        }
	}

    protected function callback_param( $name, $param, $default ) {
        $property = "{$name}_callback_${param}";
        return
            property_exists( $this, $property )
            ? intval( $this->{$property} )
            : $default;
    }


    /************************************************************************
     *
     *	Options
     *
     ************************************************************************/

    private $option_page = 'issue-taxomony';
    private $option_base = 'issue_taxomony';
    private $option_name = 'issue_taxonomy_options';

    function get_option( $name ) {
        $options = get_option( $this->option_name );

        if ( empty( $options ) )
            return NULL;
        if ( array_key_exists( $name, $options ) )
            return $options[$name];

        return NULL;
    }

    function has_option( $name ) {
        return !empty( $this->get_option( $name ) );
    }

    /*************************************************************
     *
     * Registers the `issue` taxonomy
     *
     *************************************************************/

    function do_init_action_register_editorial() {
        $post_types = $this->get_option( 'post_types' );

        if ( ! empty( $post_types ) ) {
            $registered_post_types = get_post_types( [ 'public' => true ], 'names' );

            $post_types = array_filter( $post_types,
                                        fn ( $v ) => !empty( $v ) && in_array( $v, $registered_post_types)
            );
        }
        if ( empty( $post_types ) ) {
            $post_types = $this->post_types;
        }

        register_taxonomy( $this->taxonomy, $post_types, [
            'hierarchical'          => false,
            'public'                => false,
            'show_in_nav_menus'     => true,
            'show_ui'               => true,
            'show_admin_column'     => true,
            'query_var'             => true,
            'rewrite'               => true,
            'capabilities'          => [
                'manage_terms' => 'edit_posts',
                'edit_terms'   => 'edit_posts',
                'delete_terms' => 'edit_posts',
                'assign_terms' => 'edit_posts',
            ],
            'labels'                => [
                'name'                       => __( 'Issues', 'issues-taxonomy' ),
                'singular_name'              => _x( 'Issue', 'taxonomy general name', 'issues-taxonomy' ),
                'search_items'               => __( 'Search Issues', 'issues-taxonomy' ),
                'popular_items'              => __( 'Popular Issues', 'issues-taxonomy' ),
                'all_items'                  => __( 'All Issues', 'issues-taxonomy' ),
                'parent_item'                => __( 'Parent Issue', 'issues-taxonomy' ),
                'parent_item_colon'          => __( 'Parent Issue:', 'issues-taxonomy' ),
                'edit_item'                  => __( 'Edit Issue', 'issues-taxonomy' ),
                'update_item'                => __( 'Update Issue', 'issues-taxonomy' ),
                'view_item'                  => __( 'View Issue', 'issues-taxonomy' ),
                'add_new_item'               => __( 'Add New Issue', 'issues-taxonomy' ),
                'new_item_name'              => __( 'New Issue', 'issues-taxonomy' ),
                'separate_items_with_commas' => __( 'Separate Issues with commas', 'issues-taxonomy' ),
                'add_or_remove_items'        => __( 'Add or remove Issues', 'issues-taxonomy' ),
                'choose_from_most_used'      => __( 'Choose from the most used Issues', 'issues-taxonomy' ),
                'not_found'                  => __( 'No Issues found.', 'issues-taxonomy' ),
                'no_terms'                   => __( 'No Issues', 'issues-taxonomy' ),
                'menu_name'                  => __( 'Issues', 'issues-taxonomy' ),
                'items_list_navigation'      => __( 'Issues list navigation', 'issues-taxonomy' ),
                'items_list'                 => __( 'Issues list', 'issues-taxonomy' ),
                'most_used'                  => _x( 'Most Used', 'issue', 'issues-taxonomy' ),
                'back_to_items'              => __( '&larr; Back to Issues', 'issues-taxonomy' ),
            ],
            'show_in_rest'          => true,
            'rest_base'             => $this->taxonomy,
            'rest_controller_class' => 'WP_REST_Terms_Controller',

            'update_count_callback' => '_update_generic_term_count',
        ] );

    }

    /*
     * Add filter on post lists on this taxonomy
     */

    protected $restrict_manage_posts_callback_arguments = 2;

    function do_restrict_manage_posts_action_issue( $post_type, $which ) {
        $tax = get_taxonomy( $this->taxonomy );

        if ( in_array( $post_type, $tax->object_type ) ) {

            $selected = '';
            if ( isset( $_REQUEST[ $this->taxonomy ] ) ) {
                $selected = $_REQUEST[ $this->taxonomy ];
            }

            wp_dropdown_categories(array(
                'show_option_all' =>  $tax->labels->all_items,
                'taxonomy'        =>  $this->taxonomy,
                'value_field'     => 'slug',
                'name'            =>  $this->taxonomy,
                'orderby'         =>  'name',
                'selected'        =>  $selected,
                'hierarchical'    =>  false,
                'show_count'      =>  true,
                'hide_empty'      =>  false,
            ) );
        }
    }

    /*
     *
     * Sets the post updated messages for the `issue` taxonomy.
     *
     * @param  array $messages Post updated messages.
     * @return array Messages for the `issue` taxonomy.
     *
     */

    function do_term_updated_messages_action_issue_updated_messages( $messages ) {

        $messages[ $this->taxonomy ] = [
            0 => '', // Unused. Messages start at index 1.
            1 => __( 'Issue added.', 'issues-taxonomy' ),
            2 => __( 'Issue deleted.', 'issues-taxonomy' ),
            3 => __( 'Issue updated.', 'issues-taxonomy' ),
            4 => __( 'Issue not added.', 'issues-taxonomy' ),
            5 => __( 'Issue not updated.', 'issues-taxonomy' ),
            6 => __( 'Issues deleted.', 'issues-taxonomy' ),
        ];

        return $messages;
    }

    /*************************************************************
     *
     * Dashboard widget
     *
     *************************************************************/

    function do_wp_dashboard_setup_action_status_widget() {
        wp_add_dashboard_widget(
            'streetart_dashboard_widget',
            esc_html__( 'Issue status', 'issues-taxonomy' ),
            [ $this, 'dashboard_widget_render' ]
        );
    }

    /**
     * Create the function to output the content of our Dashboard Widget.
     */
    function dashboard_widget_render() {
        // Display whatever you want to show.

        $tax = get_taxonomy( $this->taxonomy );
        $post_types = $tax->object_type;
        $terms = get_terms( [ 'taxonomy' => $this->taxonomy, 'fields' => 'all', ] );

        $counts = [];
        foreach ( $post_types as $post_type ) {
            $counts[ $post_type ] = count( get_posts( [
                'post_type' => $post_type,
                'post_status' => 'any',
                'numberposts' => -1,
                'fields' => 'ids',
            ] ) );
        }

        error_log(print_r($counts,true));

        printf( '<table width="100%%">' );

        printf( '<tr>' );
        printf("<th>Issue</th>" );
        foreach ( $post_types as $post_type ) {
            $post_type_object = get_post_type_object( $post_type );
            printf("<th align=right>%s</th>", esc_html( $post_type_object->labels->name ) );
            printf("<th align=right>%%</th>" );
        }
        printf( '</tr>' );

        foreach ( $terms as $term ) {
            printf( '<tr>' );
            printf( "<td title='%s'>%s</td>", esc_attr( $term->description ), esc_html( $term->name) );
            foreach ( $post_types as $post_type ) {
                $count = $this->get_post_count_for_term_and_post_type( $term->term_id, $post_type );
                if ( $count ) {
                    $url = add_query_arg( [
                        'issue' => $term->slug,
                        'post_type' => $post_type,
                    ], admin_url( 'edit.php' ) );
                    printf( "<td align=right><a href='%s' class='button-link'>%d</a></td>", $url, $count );
                    printf( "<td align=right>%d%%</td>", 100 * ( $count / $counts[ $post_type] ) );
                } else {
                    printf( "<td align=right>-</td>" );
                    printf( "<td align=right></td>" );
                }
            }
            printf( '</tr>' );
        }

        printf( '</table>' );
    }

    protected function get_post_count_for_term_and_post_type( $term_id, $post_type ) {
        return count ( get_posts( [
            'post_type' => $post_type,
            'post_status' => 'any',
            'numberposts' => -1,
            'tax_query' => [
                [
                    'taxonomy' => $this->taxonomy,
                      'field' => 'id',
                      'terms' => $term_id,
                  ]
              ],
              'fields' => 'ids',
          ] )
        );
    }

    /************************************************************************
     *
     *	Settings
     *
     ************************************************************************/

    /* top level menu */
    function do_admin_menu_action() {
        // add top level menu page
        add_options_page(
            'Issues Taxonomy',
            'Issues',
            'manage_options',
            $this->option_page,
            [ $this, 'settings_page_html' ]
        );
    }

    /* top level menu: callback functions */
    function settings_page_html() {
        // check user capabilities
        if ( ! current_user_can( 'manage_options' ) ) {
            return;
        }

        // check if the user have submitted the settings
        // wordpress will add the "settings-updated" $_GET parameter to the url
        if ( isset( $_GET['settings-updated'] ) ) {
            add_settings_error( $this->option_page, 'settings_saved', __( 'Settings Saved', 'issue-taxomony' ), 'updated' );
        }

        print( '<div class="wrap">' );
        printf( '<h1>%s</h1>', esc_html( get_admin_page_title()  ) );
        print( '<form action="options.php" method="post">' );

        settings_fields( $this->option_page );
        do_settings_sections( $this->option_page );
        submit_button( 'Save Settings' );

        print( '</form>' );
        print( '</div>' );
    }



    /* custom option and settings */
    function do_admin_init_action() {
        // register a new setting for "issue-taxonomy" page
        register_setting(
            $this->option_page,
            $this->option_name,
            [
                'sanitize_callback' => [ $this, 'settings_options_validate' ]
            ]
        );

        // register section for public pages, not logged in

        add_settings_section(
            $this->option_base . '_section_post_types',
            __( 'Post-types', 'issues-taxonomy' ),
            [ $this, 'settings_section_post_types_html' ],
            $this->option_page
        );

        add_settings_field(
            $this->option_base . '_post_types',
            __( 'Post-types', 'issues-taxonomy' ),
            [$this, 'settings_field_post_types_html'],
            $this->option_page,
            $this->option_base . '_section_post_types',
            [
                'field' => 'post_types',
                'label_for' => $this->option_base . '_post_types',
                'settings' => $this->option_name,
                'default' => [],
            ]
        );
    }

    /**
     * custom option and settings:
     * callback functions
     */

    function settings_options_validate( $input = NULl ) {
        return $input;
    }

    function settings_section_post_types_html( $args ) {
        echo __( '<p>Select the post-types you want to issues taxonomy to apply to.</p>' );
    }

    function settings_field_post_types_html( $args ) {
        $value = $this->get_option( $args['field'] ) ?? $args['default'];

        $post_types = get_post_types( [ 'public' => true ], 'objects' );

        printf( '<select multiple id="%s_%s" name="%s[%s][]">' . PHP_EOL,
                esc_attr( $args['settings'] ),
                esc_attr( $args['field'] ),
                esc_attr( $args['settings'] ),
                esc_attr( $args['field'] )
        );

        foreach ( $post_types as $post_type ) {
            $labels = get_post_type_labels( $post_type );

            printf( '<option value="%s" %s>%s</option>' . PHP_EOL,
                    esc_attr( $post_type->name ),
                    ( in_array( $post_type->name, $value ) ? 'selected' : '' ),
                    esc_html( $labels->name )
             );
        }
        printf( '</select>' . PHP_EOL );

        if ( isset( $args['validator'] ) and is_callable( $args['validator'] ) and !empty( $value ) and !empty( $args['error'] ) ) {
            $valid = call_user_func( $args['validator'], $value );
            if ( !$valid )
                printf( '<div style="color:red">%s</div>', $args['error'] );
        }
    }
}


global $RS_IssuesPlugin;
$RS_IssuesPlugin = new IssuesPlugin();
